/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef soma_gfx_array
#define soma_gfx_array

#include "sGFX_cube.hpp"

//**************************************************************************

/** A class for holding SomaCube objects. */
class SomaCubeArray{
public:
  /** capacity of this object ... capacity is exhausted if size() == capacity */
  const unsigned int capacity;
private:
  SomaCube** cstack;
  unsigned int csize;
  
public:
  
  /** Create a SomaCubeArray with a given capacity. */
  SomaCubeArray(const unsigned int capacity);
  
  /** Create a SomaCubeArray as copy of another. */
  SomaCubeArray(const SomaCubeArray& other);
  
  /** Destructor, destroys all remaining cubes and the object itself. */
  ~SomaCubeArray();
  
  /** Create a new cube with given position and color, if there is still 
      space in the array. */
  void insert(const SomaVector<double>& p,const SomaVector<double>& c);
  
  /** Delete a cube at a given position in the array, if it exists.*/
  void remove(unsigned int pos);
  
  /** remove all inserted cubes */
  void clear();
  
  /** Returns a pointer to the cube at position pos in the array, or null
      if there is not cube at this position */
  const SomaCube * get(const unsigned int pos);
  
  /** How many cubes are currently in the array ? Valid cubes are always at 
      positions 0...size-1. */
  inline unsigned int size(){return csize;};
};

//**************************************************************************

#endif

