/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
 
#include "sGFX_single.hpp"
#include "sGFX_cube.hpp"

//**************************************************************************

const double SomaSingleGFX::zoomrange(10);
const double SomaSingleGFX::zoomstart(-30);
const double SomaSingleGFX::safedist(20);

//**************************************************************************

SomaSingleGFX::SomaSingleGFX(SomaProb *const p):pbl(p),cubes(p->get_max_blocks()){
  dist = zoomstart;
}

//**************************************************************************

SomaSingleGFX::SomaSingleGFX(SomaProb *const p,const SomaSingleGFX *const other):
                             pbl(p),cubes(p->get_max_blocks()),
                             rotation(other->rotation){
  dist = other->dist;
}

//**************************************************************************

SomaSingleGFX::~SomaSingleGFX(){}

//**************************************************************************

void SomaSingleGFX::resize(const unsigned int width_in,
                           const unsigned int height_in){
  width = width_in;
  height = height_in;
  
  w = width;
  h = height;
  
  if(w<h){
    h = h/w*0.2;
    w = 0.2;
  }else{
    w=w/h*0.2;
    h=0.2;
  }
  
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glFrustum(-w, w, -h, h, 0.5, 0.5+2*(zoomrange+safedist)); 
}

//**************************************************************************
  
void SomaSingleGFX::draw(){
    glViewport(0,0,width,height);    
    
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
        
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    
    glTranslated(0,0,dist);
    glMultMatrixd(rotation.mtr);
    
    display(GL_RENDER);
}

//**************************************************************************
