/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

//**************************************************************************

#ifndef soma_gfx_solution
#define soma_gfx_solution

#include "s_vector.hpp"
#include "s_problem.hpp"

#include "sGFX_single.hpp"
#include "sGFX_cubearray.hpp"

class SomaSolGFX:public SomaSingleGFX{
private:
  /// scaling parameter
  double scale;
  /// whic solution are we currently displaying
  unsigned int solution;
  
  /// create drawable cubes from problem 
  void initCubes();   
  
  /// render object using opengl
  void display(const GLenum mode);
  
public:
  
  /** constructor, create object from a given problem */
  SomaSolGFX(SomaProb *const p);
  
  /** constructor, create object from a given problem, keeping
      rotation and distance parameter from another gfx object */
  SomaSolGFX(SomaProb *const p,const SomaSingleGFX *const gfx);
  
  /** destructor */
  ~SomaSolGFX();

  /** notify the object of a mouseclick */
  void mouseClick(const unsigned int x,const unsigned int y,
                  const MOUSEBUTTON button);
 
  /** notify the object of a mousemove */
  void mouseMove(const int deltax,const int deltay,
                 const MOUSEBUTTON button);
  
  /** display next solution */
  void nextSolution();
  
  /** display previous solution */
  void prevSolution();
  
  /** get current displayed solution */
  inline unsigned int getSolution()const{return solution;}
};

//**************************************************************************

#endif

