/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
#include <iostream>
 
#include "s_knapsack.hpp"

//**************************************************************************
  
SomaKnapsack::SomaKnapsack(const unsigned int *const a_in,
                           const unsigned int b_in,
                           const unsigned int max_dim_in):max_dim(max_dim_in){

#ifdef SOMA_DEBUG
  if(b_in == 0) std::cerr << "b not > 0 in SomaKnapsack::SomaKnapsack()." 
                          << std::endl;
  for(unsigned int i=0;i<max_dim;i++){
    if(a_in[i] == 0) std::cerr << "a[" << i << "] not > 0 in " 
                               << "SomaKnapsack::SomaKnapsack()." << std::endl; 
  }
#endif

  dim = max_dim;
  b = b_in;
  a = new unsigned int[max_dim];
  x = new unsigned int[max_dim];
  
  for(unsigned int i=0;i<max_dim;i++) a[i] = a_in[i];
  
  restart = 1;
}

//**************************************************************************

SomaKnapsack::~SomaKnapsack(){
  delete[] x;
  delete[] a;
}

//**************************************************************************

void SomaKnapsack::set_b(const unsigned int b_in){
  b = b_in;
  restart = 1;
}

//**************************************************************************
  
void SomaKnapsack::set_dim(const unsigned int dim_in){
  dim = dim_in < max_dim ? dim_in : dim;
  restart = 1;
}

//**************************************************************************

const unsigned int* SomaKnapsack::next_solution(){  

  if(!restart) goto jump_in;

  restart = 0;
  
  head = tail = pos = 0;
  up = 1;
  for(unsigned int i=0;i<dim;i++){
    tail += a[i];
    x[i] = 0;
  }
  
  for(;;){
  
    if(!up && pos == 0 && x[pos] == 1){
      break;
    }else if(pos == dim){ 	 
      up = 0; 	 
      pos--; 	 
      continue;
    }else if(up){
      tail -= a[pos];
      
      if(head+tail < b){
        up = 0;
        continue;
      }
      
      pos++;
      continue;
    }else if(x[pos] == 0){
      x[pos] = 1;
      head += a[pos];
      
      if(head >= b){
        if(head == b) return x;
        jump_in:
        head -= a[pos];
        tail += a[pos];
        x[pos] = 0;
        if(pos == 0) break;
        pos--;
        up = 0;
        continue;
      }
      
      up = 1;
      pos++;
      continue;
    }else{
      tail += a[pos];
      head -= a[pos];
      x[pos] = 0;
      pos--;
      continue;
    }
     
  };
  
  restart = 1;
  return 0;  
}
 
//**************************************************************************

