/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef soma_knapsack
#define soma_knapsack

//**************************************************************************

/** This class finds all solutions of a'x = b with a from N^n and b from N
    such that x in {0,1}^n for strictly positive a and b. This is done through
    a simple branch and bound type of algorithm. */
class SomaKnapsack{
private:
  unsigned int *a,*x;
  unsigned int b,dim,head,tail,pos,up,restart;
  
public:
  /** The maximum dimension the instance can handle. */
  const unsigned int max_dim;
  
  /** Constructor. a_in has to be a stricktly positive vector, whose dimension
      is given as max_dim_in. b_in hast to be stricktly positive too. */
  SomaKnapsack(const unsigned int *const a_in,const unsigned int b_in,
               const unsigned int max_dim_in);
  /** Destructor */
  ~SomaKnapsack();

  /** Set another right hand side b. This will reset solving. */  
  void set_b(const unsigned int b_in);
  
  /** Reduce the dimension, that is only the first dim_in entrys of a will
      be used. You can only set dimensions less or equal than max_dim, 
      otherwise the dimension will not change. Solutions will still have the
      dimension initially given, however the entrys highter than dim_in-1 are
      undefined. This will reset solving. */ 
  void set_dim(const unsigned int dim_in);

  /** Finds and returns a new solution. NULL is returned if there is no
      solution left. Upon the next call solutions will be reported from the
      start again. */
  const unsigned int* next_solution();
};

//**************************************************************************

#endif
