/*  Somatic - A soma puzzle solver.
 *  Copyright (C) 2003 Marc Moerig
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <iostream>
#include <fstream>
#include "../source/s_problem.hpp"
#include "../source/s_solve.hpp"

//**************************************************************************

int main(int argc, char *argv[]){
  
  if(argc < 3){
    std::cout << "usage: " << argv[0] 
              << " [input file] [output file]" << std::endl;
    return -1;
  }
  
  std::ifstream istrm(argv[1]);
  if(!istrm){
    std::cout << "Could not open '" << argv[1] 
              << "' for reading." << std::endl;
    return -1;
  }
  
  SomaProb* prb = SomaProb::create_problem(istrm);
  istrm.close();
  
  if(!prb) return -1;
  
  SomaSolve *solver = SomaSolve::createSolver(prb);
  if(solver == 0){
    std::cerr << "Problem to big, solver not startet." << std::endl;
  }else{
    solver->solve();
    delete solver;
  }
  
  if(prb->is_solved()){
    std::cout << std::endl << "Solved!" << std::endl;
  }else{
    std::cout << std::endl << "No solution !" << std::endl; 
  }
  
  std::ofstream ostrm(argv[2]);
  if(!ostrm){
    std::cerr << "Could not open '" << argv[2] << "' for writing." << std::endl;
    std::cerr << *prb;
  }else{
    ostrm << *prb;
    ostrm.close();
  }
  
  delete prb;
  
  return 0;
}

//**************************************************************************
